@@ -10,12 +10,11 @@
 #define WLR_TYPES_WLR_SCREENCOPY_V1_H
 
 #include <stdbool.h>
-#include <wayland-server.h>
+#include <wayland-server-core.h>
 #include <wlr/types/wlr_box.h>
 
 struct wlr_screencopy_manager_v1 {
 	struct wl_global *global;
-	struct wl_list resources; // wl_resource
 	struct wl_list frames; // wlr_screencopy_frame_v1::link
 
 	struct wl_listener display_destroy;
@@ -27,29 +26,41 @@ struct wlr_screencopy_manager_v1 {
 	void *data;
 };
 
+struct wlr_screencopy_v1_client {
+	int ref;
+	struct wlr_screencopy_manager_v1 *manager;
+	struct wl_list damages;
+};
+
 struct wlr_screencopy_frame_v1 {
 	struct wl_resource *resource;
-	struct wlr_screencopy_manager_v1 *manager;
+	struct wlr_screencopy_v1_client *client;
 	struct wl_list link;
 
 	enum wl_shm_format format;
+	uint32_t fourcc;
 	struct wlr_box box;
 	int stride;
 
 	bool overlay_cursor, cursor_locked;
 
-	struct wl_shm_buffer *buffer;
+	bool with_damage;
+
+	struct wl_shm_buffer *shm_buffer;
+	struct wlr_dmabuf_v1_buffer *dma_buffer;
+
 	struct wl_listener buffer_destroy;
 
 	struct wlr_output *output;
-	struct wl_listener output_swap_buffers;
+	struct wl_listener output_precommit;
+	struct wl_listener output_commit;
+	struct wl_listener output_destroy;
+	struct wl_listener output_enable;
 
 	void *data;
 };
 
 struct wlr_screencopy_manager_v1 *wlr_screencopy_manager_v1_create(
 	struct wl_display *display);
-void wlr_screencopy_manager_v1_destroy(
-	struct wlr_screencopy_manager_v1 *screencopy);
 
 #endif
